#!/usr/bin/env python3
# -*- coding: utf-8 -*- Time-stamp: <2023-04-18 15:38:34 sander>

# Rolf Sander, 2023-...

##############################################################################

import os, sys, shutil
assert sys.version_info >= (3, 6)
# from glob import glob
# import re # regexp
# from ast import literal_eval
# import decimal
import numpy as np
import matplotlib.pyplot as plt

HLINE  = '-' * 78
HLINE2 = '*' * 78

MH  =   1.00794E-3 # kg/mol
MO  =  15.9994E-3 # kg/mol
MCl =  35.453E-3 # kg/mol

MH2O = MH*2.+MO # kg/mol

rhoH2O  = 997. # kg/m3
atm     = 101325.
mmHg    = atm/760.
cH2O    = rhoH2O/MH2O # mol/m3

CtoK    = 273.15
Rgas    = 8.314
Tstp    = 273.15  # standard temperature
p0      = 101325.    # standard pressure

alpha_TO_HscpSI = 1./(Rgas*Tstp) # 4.403e-4

def rhoH2O(T):
    # rearranged from ref3802 Bettin and Spieweck (1990) with T in K,
    # valid from 0 C to 150 C.
    A0 = -2267.982655
    A1 = -5.483358159
    A2 = 0.13455975747
    A3 = -3.715682278E-4
    A4 = 4.896628144E-7
    A5 = -2.8103006E-10
    B  = 0.016887236
    rhoH2O = (A0 + A1*T + A2*T**2 + A3*T**3 + A4*T**4 + A5*T**5) / (1+B*(T-CtoK))
    return rhoH2O # in kg/m3
       
def psat(T):
    # ref3803 Buck eqn (4a) with T in K, curve ew6 from Tab. 2, valid from 0 to 100 C
    aa = 611.21
    bb = 19.6377
    cc = -17.58
    dd = 254.4
    psat = aa * np.exp( (bb-T/dd) * (T-CtoK) / (T+cc) )
    return psat # in Pa

##############################################################################

