#!/usr/bin/env python3
# -*- coding: utf-8 -*- Time-stamp: <2022-01-15 15:28:30 sander>

'''
parse henry.f90 and work on function parameters
Author: Rolf Sander, 2022-...
'''

import os, sys
assert sys.version_info >= (3, 6)
import re

VERBOSE = False

if __name__ == '__main__':

    regexp = re.compile(
        '^(.*)CALL Species\(("[^"]*"), *("[^"]*"), *("[^"]*"), *("[^"]*"), *("[^"]*")\)(.*)')
    # split long lines in henry.f90:
    with open('../henry.f90') as f_in, open('henry_NEW.f90', 'w') as f_out:
        for line in f_in:
            result = regexp.search(line)
            if result is not None:
                prefix   = result.group(1)
                formula  = result.group(2)
                casrn    = result.group(3)
                inchikey = result.group(4)
                iupac    = result.group(5)
                trivial  = result.group(6)
                comment  = result.group(7)
                if (VERBOSE):
                    print(f'prefix   = {result.group(1)}')
                    print(f'formula  = {result.group(2)}')
                    print(f'casrn    = {result.group(3)}')
                    print(f'inchikey = {result.group(4)}')
                    print(f'iupac    = {result.group(5)}')
                    print(f'trivial  = {result.group(6)}')
                    print(f'comment  = {result.group(7)}\n')
                # for formatting reasons, add comma here already:
                formula2  = formula  + ','
                inchikey2 = inchikey + ','
                # create modified CALL Species command:
                line_new = f'{prefix}CALL Species({formula2:34} {casrn:>15}, {inchikey2:30} {iupac}, {trivial}){comment}\n'
            else:
                line_new = line
            f_out.write(line_new)

##############################################################################
