#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pandas as pd
import re
import os, sys
from subprocess import PIPE, run
import urllib.request

def grep_i(searchstring, filename):
    # suffix _i indicates grep option '-i' for case insensitive
    results = []
    with open(filename) as f:
        for line in f:
            if (searchstring.lower() in line.lower()):
                results.append(line.strip())
    return results

def get_inchikey_from_henry_f90():
    result = grep_i('"'+casrn+'"', '../../henry.f90')
    if (result):
        result2 = re.search('"([A-Z]{14}-[A-Z]{10}-[A-Z])"', result[0])
        if (result2):
            return result2.group(1)
        else:
            return '??????????????-??????????-?'
    else:
        return '??????????????-??????????-?'

def get_inchikey_from_smiles(smiles):
    command = 'echo "%s" | babel -xw -ismi -oinchikey' % (smiles)
    output = run(command, shell=True, stdout=PIPE, stderr=PIPE, universal_newlines=True)
    inchikey = output.stdout.rstrip('\n')
    return inchikey

def create_CALL_Species(species, casrn, inchikey):
    print(f'NEW: {N:>4} {casrn:>14} {inchikey} {species}')

if __name__ == '__main__':

    # https://pandas.pydata.org/docs/reference/api/pandas.read_csv.html
    df = pd.read_csv('ref2546.csv', skipinitialspace=True)

    f_dat  = open('../../input/ref2546.dat', 'w')

    for index, row in df.iterrows():

        logHexp = row['logHexp']
        GROMHE  = row['GROMHE']
        SPARC   = row['SPARC']
        HWIN    = row['HWIN']
        smiles  = row['SMILES']
        casrn   = row['CASRN']
        subset  = row['Subset']
        ref     = row['Ref']

        # skip because the SMILES define nitrooxy compounds but the
        # experimental values are for nitro compounds:
        if (smiles=='CCCC(ON(=O)(=O))C(ON(=O)(=O))'):      continue
        if (smiles=='CC(ON(=O)(=O))CCC(ON(=O)(=O))'):      continue
        if (smiles=='CCCCC(ON(=O)(=O))C(ON(=O)(=O))'):     continue
        if (smiles=='C(ON(=O)(=O))CCCCC(ON(=O)(=O))'):     continue
        if (smiles=='C1CC(ON(=O)(=O))C(ON(=O)(=O))CC1'):   continue
        if (smiles=='C1CC(ON(=O)(=O))CC(ON(=O)(=O))C1'):   continue
        if (smiles=='C(ON(=O)(=O))CCCCCC(ON(=O)(=O))'):    continue
        if (smiles=='C1CCC(ON(=O)(=O))C(ON(=O)(=O))CCC1'): continue
        if (smiles=='CCCCCCC(ON(=O)(=O))C(ON(=O)(=O))'):   continue
        if (smiles=='C(ON(=O)(=O))CCCCCCC(ON(=O)(=O))'):   continue
        if (smiles=='CCCCCCCCC(ON(=O)(=O))C(ON(=O)(=O))'): continue
        if (smiles=='C(ON(=O)(=O))CCCCCCCCC(ON(=O)(=O))'): continue

        # skip because the SMILES do not define the same species as in
        # the referenced paper for the experimental values:
        if (smiles=='CC(=O)CC(C)-O-C'): continue
        if (smiles=='C(O)CC-O-CC(O)'):  continue
        
        # skip because the SMILES is strange (O with 3 bonds) and casrn
        # 928-45-0 is already used in another entry:
        if (smiles=='CCCCO(N(=O)=O)C'): continue

        # add missing casrns:
        if (smiles=='CCCC(=O)(OON(=O)=O)'):                    casrn =  '27746-48-1' # HZUMMZVMNQSPFF-UHFFFAOYSA-N
        if (smiles=='CC(C)C(=O)(OON(=O)=O)'):                  casrn =  '65424-60-4' # BDNFHGUXBRZLRQ-UHFFFAOYSA-N
        if (smiles=='CC(ON(=O)(=O))CCC(ON(=O)(=O))C'):         casrn =  '99115-63-6' # ISSLCMSTXXUOEU-UHFFFAOYSA-N
        if (smiles=='C(Cl)C(Cl)CCCCCCC(Cl)C(Cl)'):             casrn = '205646-11-3' # VXBHNYIEBLRXAW-UHFFFAOYSA-N
        if (smiles=='C(Cl)C(Cl)CCCCCCCC(Cl)C(Cl)'):            casrn = '210049-49-3' # VVAAXDBMZVCVPA-UHFFFAOYSA-N
        if (smiles=='C(C=C)(=O)(OON(=O)=O)'):                  casrn = '157258-66-7' # SFKRQQJZRXQGLC-UHFFFAOYSA-N MCM:ACRPAN       
        if (smiles=='C(O)CC(ON(=O)(=O))'):                     casrn = '100502-66-7' # PTMLFFXFTRSBJW-UHFFFAOYSA-N MCM:HO1C3NO3     
        if (smiles=='CC(ON(=O)=O)(C)C'):                       casrn =    '926-05-6' # AZAKMLHUDVIDFN-UHFFFAOYSA-N MCM:TC4H9NO3     
        if (smiles=='c1c(N(=O)(=O))cc(N(=O)(=O))c(O)c1C(C)C'): casrn =  '29385-11-3' # HBYHYLBZPLCIEE-UHFFFAOYSA-N MCM:DNIPBNZOL    

        # assuming that HOCH2OOH is meant, not HOCH2OO:
        if (casrn== '27828-51-9'): casrn = '15932-89-5'

        # wrong casrn:
        if (casrn==  '633-11-8'): casrn = '20633-11-8'
        if (casrn=='8294-459-0'): casrn = '82944-59-0'
        
        # skip because cis/trans is not specified:
        if (casrn== '156-59-2'): continue # CHCl=CHCl
        # skip because the experimental value is for nitrooxyacetone, not nitroacetone:
        if (casrn== '10230-68-9'): continue
        # skip because the experimental value is for 1,4-pentanediol
        # dinitrate, not 2,4-pentanediol dinitrate:
        if (casrn== '101421-04-9'): continue

        # run babel to get inchikey from SMILES:
        inchikey = get_inchikey_from_smiles(smiles)

        # get inchikey from henry.f90:
        inchikey2 = get_inchikey_from_henry_f90()

        # add stereochemistry according to casrn:
        if (casrn=='2207-04-7'): inchikey = 'QRMPKOFEUHIBNM-ZKCHVHJHSA-N' # trans-1,4-dimethylcyclohexane
        if (casrn== '503-64-0'): inchikey = 'LDHQCZJRKDOVOX-IHWYPQMZSA-N' # isocrotonic acid
        if (casrn== '105-76-0'): inchikey = 'JBSLOWBPDRZSMB-FPLPWBNLSA-N' # dibutyl maleate
        if (casrn== '624-48-6'): inchikey = 'LDCRTTXIJACKKU-ARJAWSKDSA-N' # dimethyl maleate

        # assuming that casrn is correct and SMILES is wrong:
        if (casrn=='1712-64-7'): inchikey = 'GAPFWGOSHOCNBM-UHFFFAOYSA-N'
        if (casrn== '543-29-3'): inchikey = 'LNNXFUZKZLXPOF-UHFFFAOYSA-N'

        # consistency check:
        if (not inchikey==inchikey2):
            quoted_casrn = '"' + casrn + '"'
            print(f'{quoted_casrn:>14} "{inchikey2}" "{inchikey}" {smiles}')

        # write to f90 input file:
        quoted_casrn = '"' + casrn + '"'
        quoted_ref   = '"' + ref   + '"'
        print(f'{logHexp:5} {GROMHE:5} {SPARC:5} {HWIN:5} "{subset}" ' +
              f'{quoted_casrn:>14} "{inchikey}" {quoted_ref:14} "{smiles}"', file=f_dat)

    f_dat.close()
