#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np
import pandas as pd
import re
import os, sys
from subprocess import PIPE, run
import datetime

import ref3579_dict
get_inchikey = ref3579_dict.get_inchikey() # load dictionary

df = pd.read_excel('ref3406-sp.xls', skiprows=1)

if __name__ == '__main__':

    f_dat  = open('../../input/ref3406.dat', 'w')

    for index, row in df.iterrows():
        i       = row['No.']
        chem    = row['Comp.a']
        formula = row['Form.b']
        H       = row['HRep/Pred (atm/mol frac)']
        lgH     = row['log(HRep/Pred)d']
        if (i=='Overall'): break # end of table reached

        # skip 1,cis-2,trans-3,trans-4-tetramethylcyclopentane:
        if (chem=="1,cis-2,trans-3,trans-4-tetramethylcyclopentane"):
            print('Skipping 1,cis-2,trans-3,trans-4-tetramethylcyclopentane')
            print('because inchikey is unknown and casrn is questionable.')
            continue
        # skip vinylcyclohexene:
        if (chem=='vinylcyclohexene'):
            print('Skipping vinylcyclohexene because it is unclear if this is')
            print('1-vinylcyclohexene or 4-vinylcyclohexene.')
            continue

        inchikey = get_inchikey.get(chem,None)
        if (not inchikey):
            if (chem=="adipic acid"):                          inchikey = "WNLRTRBMVRJNCN-UHFFFAOYSA-N"
            if (chem=="sec-butyl formate"):                    inchikey = "OAEQYDZVVPONKW-UHFFFAOYSA-N"
            if (chem=="m-cresol"):                             inchikey = "RLSSMJSEOOYNOY-UHFFFAOYSA-N"
            if (chem=="2-decanone"):                           inchikey = "ZAJNGDIORYACQU-UHFFFAOYSA-N"
            if (chem=="decyl acetate"):                        inchikey = "NUPSHWCALHZGOV-UHFFFAOYSA-N"
            if (chem=="diethyl carbonate"):                    inchikey = "OIFBSDVPJOWBCH-UHFFFAOYSA-N"
            if (chem=="2,6-dimethylnaphthalene"):              inchikey = "YGYNBBAUIYTWBF-UHFFFAOYSA-N"
            if (chem=="2-dodecanone"):                         inchikey = "LSKONYYRONEBKA-UHFFFAOYSA-N"
            if (chem=="1-eicosanol"):                          inchikey = "BTFJIXJJCSYFAL-UHFFFAOYSA-N"
            if (chem=="ethyl formate"):                        inchikey = "WBJINCZRORDGAQ-UHFFFAOYSA-N"
            if (chem=="hexadecanoic acid"):                    inchikey = "IPCSVZSSVZVIGE-UHFFFAOYSA-N"
            if (chem=="isobutyl isobutyrate"):                 inchikey = "RXGUIWHIADMCFC-UHFFFAOYSA-N"
            if (chem=="isopropyl acetate"):                    inchikey = "JMMWKPVZQRWMSS-UHFFFAOYSA-N"
            if (chem=="1-methyl-2-ethyl-1-hexanol"):           inchikey = "MMQDVLWWGWJSFS-UHFFFAOYSA-N"
            if (chem=="2-nonanone"):                           inchikey = "VKCYHJWLYTUGCC-UHFFFAOYSA-N"
            if (chem=="undecyl acetate"):                      inchikey = "CKQGCFFDQIFZFA-UHFFFAOYSA-N"
            if (chem=="acenaphthene"):                         inchikey = "CWRYPZZKDGJXCA-UHFFFAOYSA-N"
            if (chem=="acrolein"):                             inchikey = "HGINCPLSRVDWNT-UHFFFAOYSA-N"
            if (chem=="p-aminoazobenzene"):                    inchikey = "QPQKUYVSJWQSDY-UHFFFAOYSA-N"
            if (chem=="1,2,3-benzenetriol"):                   inchikey = "WQGWDDDVZFFDIG-UHFFFAOYSA-N"
            if (chem=="bromoform"):                            inchikey = "DIKBFYAXUHHXCS-UHFFFAOYSA-N"
            if (chem=="1,3 butadiene"):                        inchikey = "KAKZBPTYRLMSJV-UHFFFAOYSA-N"
            if (chem=="butanol"):                              inchikey = "LRHPLDYGYMQRHN-UHFFFAOYSA-N"
            if (chem=="butyl acetate"):                        inchikey = "DKPFZGUDAPQIHT-UHFFFAOYSA-N"
            if (chem=="butyl acrylate"):                       inchikey = "CQEYYJKEWSMYFG-UHFFFAOYSA-N"
            if (chem=="chloroform"):                           inchikey = "HEDRZPFGACZZDS-UHFFFAOYSA-N"
            if (chem=="p-cresol"):                             inchikey = "IWDCLRJOBJJRNH-UHFFFAOYSA-N"
            if (chem=="di-propylamine"):                       inchikey = "WEHWNAOGRSTTBQ-UHFFFAOYSA-N"
            if (chem=="dibutyl ether"):                        inchikey = "DURPTKYDGMDSBL-UHFFFAOYSA-N"
            if (chem=="dichlorofluoromethane"):                inchikey = "UMNKXPULIDJLSU-UHFFFAOYSA-N"
            if (chem=="m-difluorobenzene"):                    inchikey = "UEMGWPRHOOEKTA-UHFFFAOYSA-N"
            if (chem=="1,1-difluoroethane"):                   inchikey = "NPNPZTNLOVBDOC-UHFFFAOYSA-N"
            if (chem=="difluoromethane"):                      inchikey = "RWRIWBAIICGTTQ-UHFFFAOYSA-N"
            if (chem=="3,3-dimethyl-1-butanol"):               inchikey = "DUXCSEISVMREAX-UHFFFAOYSA-N"
            if (chem=="3,4-dimethyl-2-pentanol"):              inchikey = "SODKTKJZYBOMMT-UHFFFAOYSA-N"
            if (chem=="1,3-dimethylnaphthalene"):              inchikey = "QHJMFSMPSZREIF-UHFFFAOYSA-N"
            if (chem=="1,4-dimethylnaphthalene"):              inchikey = "APQSQLNWAIULLK-UHFFFAOYSA-N"
            if (chem=="m-dinitrobenzene"):                     inchikey = "WDCYWAQPCXBPJA-UHFFFAOYSA-N"
            if (chem=="2-ethyl-1-hexanol"):                    inchikey = "YIWUKEYIRIRTPP-UHFFFAOYSA-N"
            if (chem=="ethyl acetate"):                        inchikey = "XEKOWRVHYACXOJ-UHFFFAOYSA-N"
            if (chem=="ethyl acrylate"):                       inchikey = "JIGUQPWFLRLWPJ-UHFFFAOYSA-N"
            if (chem=="1-ethylpropyl acetate"):                inchikey = "PBKYSIMDORTIEU-UHFFFAOYSA-N"
            if (chem=="1-ethylpropyl formate"):                inchikey = "YMJOAYHERILFIM-UHFFFAOYSA-N"
            if (chem=="furfuryl alcohol"):                     inchikey = "XPFVYQJUAUNWIW-UHFFFAOYSA-N"
            if (chem=="heptanoic acid"):                       inchikey = "MNWFXJYAOYHMED-UHFFFAOYSA-N"
            if (chem=="3-heptanol"):                           inchikey = "RZKSECIXORKHQS-UHFFFAOYSA-N"
            if (chem=="4-heptanol"):                           inchikey = "YVBCULSIZWMTFY-UHFFFAOYSA-N"
            if (chem=="p-hydroxybenzaldehyde"):                inchikey = "RGHHSNMVTDWUBI-UHFFFAOYSA-N"
            if (chem=="isobutyl acrylate"):                    inchikey = "CFVWNXQPGQOHRJ-UHFFFAOYSA-N"
            if (chem=="isobutyric acid"):                      inchikey = "KQNPFQTWMSNSAP-UHFFFAOYSA-N"
            if (chem=="isopropyl formate"):                    inchikey = "RMOUBSOVHSONPZ-UHFFFAOYSA-N"
            if (chem=="malononitrile"):                        inchikey = "CUONGYYJJVDODC-UHFFFAOYSA-N"
            if (chem=="methane"):                              inchikey = "VNWKTOKETHGBQD-UHFFFAOYSA-N"
            if (chem=="5-methyl-1-heptanol"):                  inchikey = "KFARNLMRENFOHE-UHFFFAOYSA-N"
            if (chem=="3-methyl-1-pentanol"):                  inchikey = "IWTBVKIGCDZRPL-UHFFFAOYSA-N"
            if (chem=="4-methyl-2-heptanol"):                  inchikey = "GUHWHNUGIGOSCN-UHFFFAOYSA-N"
            if (chem=="6-methyl-2-heptanol"):                  inchikey = "FCOUHTHQYOMLJT-UHFFFAOYSA-N"
            if (chem=="5-methyl-2-hexanol"):                   inchikey = "ZDVJGWXFXGJSIU-UHFFFAOYSA-N"
            if (chem=="methyl butanoate"):                     inchikey = "UUIQMZJEGPQKFD-UHFFFAOYSA-N"
            if (chem=="1-methylbutyl acetate"):                inchikey = "GQKZRWSUJHVIPE-UHFFFAOYSA-N"
            if (chem=="methyl tert-butyl ether"):              inchikey = "BZLVMXJERCGZMT-UHFFFAOYSA-N"
            if (chem=="methyl formate"):                       inchikey = "TZIHFWKZFHZASV-UHFFFAOYSA-N"
            if (chem=="methylisopropylamine"):                 inchikey = "XHFGWHUWQXTGAT-UHFFFAOYSA-N"
            if (chem=="nonylbenzene"):                         inchikey = "LIXVMPBOGDCSRM-UHFFFAOYSA-N"
            if (chem=="o-cresol"):                             inchikey = "QWVGKYWNOKOFNN-UHFFFAOYSA-N"
            if (chem=="4-octanol"):                            inchikey = "WOFPPJOZXUTRAU-UHFFFAOYSA-N"
            if (chem=="octylamine"):                           inchikey = "IOQPZZOEVPZRBK-UHFFFAOYSA-N"
            if (chem=="pentachloroethane"):                    inchikey = "BNIXVQGCZULYKV-UHFFFAOYSA-N"
            if (chem=="1,2-propanediol"):                      inchikey = "DNIAPMSPPWPWGF-UHFFFAOYSA-N"
            if (chem=="propionaldehyde"):                      inchikey = "NBBJYMSMWIIQGU-UHFFFAOYSA-N"
            if (chem=="quinoline"):                            inchikey = "SMWDFEZZVXVKRB-UHFFFAOYSA-N"
            if (chem=="resorcinol"):                           inchikey = "GHMLBKRAJCXXBS-UHFFFAOYSA-N"
            if (chem=="terephthalic acid"):                    inchikey = "KKEYFWRCBNTPAC-UHFFFAOYSA-N"
            if (chem=="tetradecanal"):                         inchikey = "UHUFTBALEZWWIH-UHFFFAOYSA-N"
            if (chem=="2-tetradecanol"):                       inchikey = "BRGJIIMZXMWMCC-UHFFFAOYSA-N"
            if (chem=="2-tetradecanone"):                      inchikey = "POQLVOYRGNFGRM-UHFFFAOYSA-N"
            if (chem=="tetrahydrothiophene"):                  inchikey = "RAOIDOHSFRTOEL-UHFFFAOYSA-N"
            if (chem=="p-toluidine"):                          inchikey = "RZXMPPFPUUCRFN-UHFFFAOYSA-N"
            if (chem=="2-tridecanone"):                        inchikey = "CYIFVRUOHKNECG-UHFFFAOYSA-N"
            if (chem=="2,4,4-trimethyl-1-pentanol"):           inchikey = "ZNRVRWHPZZOTIE-UHFFFAOYSA-N"
            if (chem=="1,cis-3,cis-5-trimethylcyclohexane"):   inchikey = "ODNRTOSCFYDTKF-AYMMMOKOSA-N"
            if (chem=="1,cis-3,trans-5-trimethylcyclohexane"): inchikey = "ODNRTOSCFYDTKF-FBJIGQNJSA-N"
            if (chem=="2,4-xylenol"):                          inchikey = "KUFFULVDNCHOFZ-UHFFFAOYSA-N"
            if (chem=="3,4-xylenol"):                          inchikey = "YCOXTKKNXUZSKD-UHFFFAOYSA-N"

        print(f'{i:4} {lgH:8} "{inchikey}" "{chem}"', file=f_dat)
            
    f_dat.close()
