#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np
import pandas as pd
import re
import os, sys
from subprocess import PIPE, run
import datetime

# pip3 install xlrd openpyxl
df = pd.read_excel('ref3579-perscomm.xlsx', engine='openpyxl')

df2 = pd.read_excel('ref3579.xls', skiprows=2)
df2 = df2.rename(columns={'Unnamed: 0': 'No.', 'Unnamed: 1': 'NAME', 'Unnamed: 5': 'Status'})

# for columns in both data frames, the suffixes _x and _y are appended
# https://pandas.pydata.org/docs/reference/api/pandas.DataFrame.merge.html
df_merged = pd.merge(df, df2, on=['No.'])

def grep_i(searchstring, filename):
    # suffix _i indicates grep option '-i' for case insensitive
    results = []
    with open(filename) as f:
        for line in f:
            if (searchstring in line):
                results.append(line.strip())
    return results

def get_casrn_from_henry_f90():
    if (not inchikey): return ''
    result = grep_i('"'+inchikey+'"', '../../henry.f90')
    if (result):
        return re.sub('.*"([^"]*)", *"[-A-Z]+".*', r'\1', result[0], flags=re.IGNORECASE)
    return ''

def get_inchikey_from_henry_f90():
    result = grep_i('"'+casrn+'"', '../../henry.f90')
    if (result):
        return re.sub('.*, "([^"]*)".*', r'\1', result[0])
    else:
        return ''

def get_inchikey_from_smiles(smiles):
    command = 'echo "%s" | babel -xw -ismi -oinchikey' % (smiles)
    output = run(command, shell=True, stdout=PIPE, stderr=PIPE, universal_newlines=True)
    inchikey = output.stdout.rstrip('\n')
    # correct some inchikeys that were not calculated correctly because
    # SMILES in csv do not include stereoisomers:
    #if(mcmname=='BCARY'):
    #    inchikey = 'NPNUFJAVOOONJE-IOMPXFEGSA-N' # 87-44-5
    return inchikey

if __name__ == '__main__':

    f_dat  = open('../../input/ref3579.dat', 'w')
    f_dict = open('ref3579-dict.py', 'w')
    f_f90  = open('ref3579-raw.f90', 'w')

    print('# created automatically by ref3579.py, DO NOT EDIT!', file=f_dict)
    print('def get_inchikey():', file=f_dict)
    print('    return {', file=f_dict)
    namedict = []
    
    for index, row in df_merged.iterrows():
        i       = row['No.']
        #if (i>30): break #qqq only a few lines for testing
        chem    = row['NAME']
        chemstr = f'"{chem}"'
        if (not chem==row['MolID']):
            print(i, chem, row['MolID'])
        H_yaws  = row['Yaws_x']
        if (not H_yaws==row['Yaws_y']):
            print(i, H_yaws, row['Yaws_y'])
        H       = row['NN-GC']
        formula = row['Formula']
        casrn   = row['CASN']
        smiles  = row['SMILES']
        inchi   = row['InChi']
        temp    = row['T [C]']

        # improve some chem names for LaTeX:
        if (chem=="sec-butylcyclopentane"):                                     chem = "{\it sec}-butylcyclopentane"
        if (chem=="tert-butylcyclopentane"):                                    chem = "{\it tert}-butylcyclopentane"
        if (chem=="4-tert-butylheptane"):                                       chem = "4-{\it tert}-butylheptane"
        if (chem=="4-tert-butyloctane"):                                        chem = "4-{\it tert}-butyloctane"
        if (chem=="di-sec-butyl ether"):                                        chem = "di-{\it sec}-butyl ether"
        if (chem=="di-tert-butyl ether"):                                       chem = "di-{\it tert}-butyl ether"
        if (chem=="1,cis-2-diethylcyclopentane"):                               chem = "1,{\it cis}-2-diethylcyclopentane"
        if (chem=="1,trans-2-diethylcyclopentane"):                             chem = "1,{\it trans}-2-diethylcyclopentane"
        if (chem=="1,cis-3-diethylcyclopentane"):                               chem = "1,{\it cis}-3-diethylcyclopentane"
        if (chem=="1,trans-3-diethylcyclopentane"):                             chem = "1,{\it trans}-3-diethylcyclopentane"
        if (chem=="m-diisopropylbenzene"):                                      chem = "$m$-diisopropylbenzene"
        if (chem=="p-diisopropylbenzene"):                                      chem = "$p$-diisopropylbenzene"
        if (chem=="1,cis-2-dimethyl-1-ethylcyclopentane"):                      chem = "1,{\it cis}-2-dimethyl-1-ethylcyclopentane"
        if (chem=="1,trans-2-dimethyl-1-ethylcyclopentane"):                    chem = "1,{\it trans}-2-dimethyl-1-ethylcyclopentane"
        if (chem=="1,cis-3-dimethyl-1-ethylcyclopentane"):                      chem = "1,{\it cis}-3-dimethyl-1-ethylcyclopentane"
        if (chem=="1,trans-3-dimethyl-1-ethylcyclopentane"):                    chem = "1,{\it trans}-3-dimethyl-1-ethylcyclopentane"
        if (chem=="2,3-dimethyl-2-tert-butyl-1-butanol"):                       chem = "2,3-dimethyl-2-{\it tert}-butyl-1-butanol"
        if (chem=="1,cis-3-dimethyl-cis-2-ethylcyclopentane"):                  chem = "1,{\it cis}-3-dimethyl-{\it cis}-2-ethylcyclopentane"
        if (chem=="1,cis-3-dimethyl-trans-2-ethylcyclopentane"):                chem = "1,{\it cis}-3-dimethyl-{\it trans}-2-ethylcyclopentane"
        if (chem=="1,trans-3-dimethyl-cis-2-ethylcyclopentane"):                chem = "1,{\it trans}-3-dimethyl-{\it cis}-2-ethylcyclopentane"
        if (chem=="3,4-dimethyl-cis-2-hexene"):                                 chem = "3,4-dimethyl-{\it cis}-2-hexene"
        if (chem=="3,4-dimethyl-trans-2-hexene"):                               chem = "3,4-dimethyl-{\it trans}-2-hexene"
        if (chem=="3,5-dimethyl-cis-2-hexene"):                                 chem = "3,5-dimethyl-{\it cis}-2-hexene"
        if (chem=="3,5-dimethyl-trans-2-hexene"):                               chem = "3,5-dimethyl-{\it trans}-2-hexene"
        if (chem=="4,4-dimethyl-cis-2-hexene"):                                 chem = "4,4-dimethyl-{\it cis}-2-hexene"
        if (chem=="4,4-dimethyl-trans-2-hexene"):                               chem = "4,4-dimethyl-{\it trans}-2-hexene"
        if (chem=="4,5-dimethyl-cis-2-hexene"):                                 chem = "4,5-dimethyl-{\it cis}-2-hexene"
        if (chem=="4,5-dimethyl-trans-2-hexene"):                               chem = "4,5-dimethyl-{\it trans}-2-hexene"
        if (chem=="5,5-dimethyl-cis-2-hexene"):                                 chem = "5,5-dimethyl-{\it cis}-2-hexene"
        if (chem=="5,5-dimethyl-trans-2-hexene"):                               chem = "5,5-dimethyl-{\it trans}-2-hexene"
        if (chem=="3,4-dimethyl-cis-2-pentene"):                                chem = "3,4-dimethyl-{\it cis}-2-pentene"
        if (chem=="3,4-dimethyl-trans-2-pentene"):                              chem = "3,4-dimethyl-{\it trans}-2-pentene"
        if (chem=="4,4-dimethyl-cis-2-pentene"):                                chem = "4,4-dimethyl-{\it cis}-2-pentene"
        if (chem=="4,4-dimethyl-trans-2-pentene"):                              chem = "4,4-dimethyl-{\it trans}-2-pentene"
        if (chem=="2,2-dimethyl-3-tert-butylhexane"):                           chem = "2,2-dimethyl-3-{\it tert}-butylhexane"
        if (chem=="1,cis-2-dimethyl-cis-3-ethylcyclopentane"):                  chem = "1,{\it cis}-2-dimethyl-{\it cis}-3-ethylcyclopentane"
        if (chem=="1,cis-2-dimethyl-trans-3-ethylcyclopentane"):                chem = "1,{\it cis}-2-dimethyl-{\it trans}-3-ethylcyclopentane"
        if (chem=="1,trans-2-dimethyl-cis-3-ethylcyclopentane"):                chem = "1,{\it trans}-2-dimethyl-{\it cis}-3-ethylcyclopentane"
        if (chem=="1,trans-2-dimethyl-trans-3-ethylcyclopentane"):              chem = "1,{\it trans}-2-dimethyl-{\it trans}-3-ethylcyclopentane"
        if (chem=="2,2-dimethyl-cis-3-hexene"):                                 chem = "2,2-dimethyl-{\it cis}-3-hexene"
        if (chem=="2,2-dimethyl-trans-3-hexene"):                               chem = "2,2-dimethyl-{\it trans}-3-hexene"
        if (chem=="2,3-dimethyl-cis-3-hexene"):                                 chem = "2,3-dimethyl-{\it cis}-3-hexene"
        if (chem=="2,3-dimethyl-trans-3-hexene"):                               chem = "2,3-dimethyl-{\it trans}-3-hexene"
        if (chem=="2,4-dimethyl-cis-3-hexene"):                                 chem = "2,4-dimethyl-{\it cis}-3-hexene"
        if (chem=="2,4-dimethyl-trans-3-hexene"):                               chem = "2,4-dimethyl-{\it trans}-3-hexene"
        if (chem=="2,5-dimethyl-cis-3-hexene"):                                 chem = "2,5-dimethyl-{\it cis}-3-hexene"
        if (chem=="2,5-dimethyl-trans-3-hexene"):                               chem = "2,5-dimethyl-{\it trans}-3-hexene"
        if (chem=="3,4-dimethyl-cis-3-hexene"):                                 chem = "3,4-dimethyl-{\it cis}-3-hexene"
        if (chem=="3,4-dimethyl-trans-3-hexene"):                               chem = "3,4-dimethyl-{\it trans}-3-hexene"
        if (chem=="1,cis-2-dimethyl-cis-4-ethylcyclopentane"):                  chem = "1,{\it cis}-2-dimethyl-{\it cis}-4-ethylcyclopentane"
        if (chem=="1,cis-2-dimethyl-trans-4-ethylcyclopentane"):                chem = "1,{\it cis}-2-dimethyl-{\it trans}-4-ethylcyclopentane"
        if (chem=="1,trans-2-dimethyl-cis-4-ethylcyclopentane"):                chem = "1,{\it trans}-2-dimethyl-{\it cis}-4-ethylcyclopentane"
        if (chem=="1,cis-3-dimethyl-cis-4-ethylcyclopentane"):                  chem = "1,{\it cis}-3-dimethyl-{\it cis}-4-ethylcyclopentane"
        if (chem=="1,cis-3-dimethyl-trans-4-ethylcyclopentane"):                chem = "1,{\it cis}-3-dimethyl-{\it trans}-4-ethylcyclopentane"
        if (chem=="1,trans-3-dimethyl-cis-4-ethylcyclopentane"):                chem = "1,{\it trans}-3-dimethyl-{\it cis}-4-ethylcyclopentane"
        if (chem=="1,trans-3-dimethyl-trans-4-ethylcyclopentane"):              chem = "1,{\it trans}-3-dimethyl-{\it trans}-4-ethylcyclopentane"
        if (chem=="dimethyl-sec-butylamine"):                                   chem = "dimethyl-{\it sec}-butylamine"
        if (chem=="dimethyl-tert-butylamine"):                                  chem = "dimethyl-{\it tert}-butylamine"
        if (chem=="trans-1,2-dimethylcyclohexane"):                             chem = "{\it trans}-1,2-dimethylcyclohexane"
        if (chem=="cis-1,3-dimethylcyclohexane"):                               chem = "{\it cis}-1,3-dimethylcyclohexane"
        if (chem=="trans-1,3-dimethylcyclohexane"):                             chem = "{\it trans}-1,3-dimethylcyclohexane"
        if (chem=="cis-1,4-dimethylcyclohexane"):                               chem = "{\it cis}-1,4-dimethylcyclohexane"
        if (chem=="cis-1,2-dimethylcyclopentane"):                              chem = "{\it cis}-1,2-dimethylcyclopentane"
        if (chem=="trans-1,2-dimethylcyclopentane"):                            chem = "{\it trans}-1,2-dimethylcyclopentane"
        if (chem=="cis-1,3-dimethylcyclopentane"):                              chem = "{\it cis}-1,3-dimethylcyclopentane"
        if (chem=="trans-1,3-dimethylcyclopentane"):                            chem = "{\it trans}-1,3-dimethylcyclopentane"
        if (chem=="2,6-dimethyt-2-octanol"):                                    chem = "2,6-dimethyl-2-octanol"
        if (chem=="3-ethyl-cis-2-hexene"):                                      chem = "3-ethyl-{\it cis}-2-hexene"
        if (chem=="3-ethyl-trans-2-hexene"):                                    chem = "3-ethyl-{\it trans}-2-hexene"
        if (chem=="4-ethyl-cis-2-hexene"):                                      chem = "4-ethyl-{\it cis}-2-hexene"
        if (chem=="4-ethyl-trans-2-hexene"):                                    chem = "4-ethyl-{\it trans}-2-hexene"
        if (chem=="ethyl-sec-butylamine"):                                      chem = "ethyl-{\it sec}-butylamine"
        if (chem=="3-ethyl-o-xylene"):                                          chem = "3-ethyl-$o$-xylene"
        if (chem=="4-ethyl-o-xylene"):                                          chem = "4-ethyl-$o$-xylene"
        if (chem=="2-ethyl-m-xylene"):                                          chem = "2-ethyl-$m$-xylene"
        if (chem=="4-ethyl-m-xylene"):                                          chem = "4-ethyl-$m$-xylene"
        if (chem=="5-ethyl-m-xylene"):                                          chem = "5-ethyl-$m$-xylene"
        if (chem=="2-ethyl-p-xylene"):                                          chem = "2-ethyl-$p$-xylene"
        if (chem=="ethyl sec-butyl ether"):                                     chem = "ethyl {\it sec}-butyl ether"
        if (chem=="1,cis-3-hexadiene"):                                         chem = "1,{\it cis}-3-hexadiene"
        if (chem=="1,trans-3-hexadiene"):                                       chem = "1,{\it trans}-3-hexadiene"
        if (chem=="1,cis-4-hexadiene"):                                         chem = "1,{\it cis}-4-hexadiene"
        if (chem=="1,trans-4-hexadiene"):                                       chem = "1,{\it trans}-4-hexadiene"
        if (chem=="cis-2,trans-4-hexadiene"):                                   chem = "{\it cis}-2,{\it trans}-4-hexadiene"
        if (chem=="cis-2,cis-4-hexadiene"):                                     chem = "{\it cis}-2,{\it cis}-4-hexadiene"
        if (chem=="trans-2,trans-4-hexadiene"):                                 chem = "{\it trans}-2,{\it trans}-4-hexadiene"
        if (chem=="meso-2,3-butanediol"):                                       chem = "{\it meso}-2,3-butanediol"
        if (chem=="2-methyl-1,cis-3-pentadiene"):                               chem = "2-methyl-1,{\it cis}-3-pentadiene"
        if (chem=="2-methyl-1,trans-3-pentadiene"):                             chem = "2-methyl-1,{\it trans}-3-pentadiene"
        if (chem=="3-methyl-1,cis-3-pentadiene"):                               chem = "3-methyl-1,{\it cis}-3-pentadiene"
        if (chem=="3-methyl-1,trans-3-pentadiene"):                             chem = "3-methyl-1,{\it trans}-3-pentadiene"
        if (chem=="1-methyl-2-sec-butylbenzene"):                               chem = "1-methyl-2-{\it sec}-butylbenzene"
        if (chem=="1-methyl-2-tert-butyl benzene"):                             chem = "1-methyl-2-{\it tert}-butyl benzene"
        if (chem=="1-methyl-cis-2-ethylcyclohexane"):                           chem = "1-methyl-{\it cis}-2-ethylcyclohexane"
        if (chem=="1-methyl-trans-2-ethylcyclohexane"):                         chem = "1-methyl-{\it trans}-2-ethylcyclohexane"
        if (chem=="1-methyl-cis-2-ethylcyclopentane"):                          chem = "1-methyl-{\it cis}-2-ethylcyclopentane"
        if (chem=="1-methyl-trans-2-ethylcyclopentane"):                        chem = "1-methyl-{\it trans}-2-ethylcyclopentane"
        if (chem=="3-methyl-cis-2-heptene"):                                    chem = "3-methyl-{\it cis}-2-heptene"
        if (chem=="3-methyl-trans-2-heptene"):                                  chem = "3-methyl-{\it trans}-2-heptene"
        if (chem=="4-methyl-cis-2-heptene"):                                    chem = "4-methyl-{\it cis}-2-heptene"
        if (chem=="4-methyl-trans-2-heptene"):                                  chem = "4-methyl-{\it trans}-2-heptene"
        if (chem=="5-methyl-cis-2-heptene"):                                    chem = "5-methyl-{\it cis}-2-heptene"
        if (chem=="5-methyl-trans-2-heptene"):                                  chem = "5-methyl-{\it trans}-2-heptene"
        if (chem=="6-methyl-cis-2-heptene"):                                    chem = "6-methyl-{\it cis}-2-heptene"
        if (chem=="6-methyl-trans-2-heptene"):                                  chem = "6-methyl-{\it trans}-2-heptene"
        if (chem=="3-methyl-cis-2-hexene"):                                     chem = "3-methyl-{\it cis}-2-hexene"
        if (chem=="3-methyl-trans-2-hexene"):                                   chem = "3-methyl-{\it trans}-2-hexene"
        if (chem=="4-methyl-cis-2-hexene"):                                     chem = "4-methyl-{\it cis}-2-hexene"
        if (chem=="4-methyl-trans-2-hexene"):                                   chem = "4-methyl-{\it trans}-2-hexene"
        if (chem=="5-methyl-cis-2-hexene"):                                     chem = "5-methyl-{\it cis}-2-hexene"
        if (chem=="5-methyl-trans-2-hexene"):                                   chem = "5-methyl-{\it trans}-2-hexene"
        if (chem=="1-methyl-cis-2-isopropylcyclopentane"):                      chem = "1-methyl-{\it cis}-2-isopropylcyclopentane"
        if (chem=="1-methyl-trans-2-isopropylcyclopentane"):                    chem = "1-methyl-{\it trans}-2-isopropylcyclopentane"
        if (chem=="3-methyl-cis-2-pentene"):                                    chem = "3-methyl-{\it cis}-2-pentene"
        if (chem=="1-methyl-cis-2-propylcyclopentane"):                         chem = "1-methyl-{\it cis}-2-propylcyclopentane"
        if (chem=="1-methyl-trans-2-propylcyclopentane"):                       chem = "1-methyl-{\it trans}-2-propylcyclopentane"
        if (chem=="1-methyl-3-sec-butylbenzene"):                               chem = "1-methyl-3-{\it sec}-butylbenzene"
        if (chem=="1-methyl-3-tert-butylbenzene"):                              chem = "1-methyl-3-{\it tert}-butylbenzene"
        if (chem=="4-methyl-3-ethyl-cis-2-pentene"):                            chem = "4-methyl-3-ethyl-{\it cis}-2-pentene"
        if (chem=="4-methyl-3-ethyl-trans-2-pentene"):                          chem = "4-methyl-3-ethyl-{\it trans}-2-pentene"
        if (chem=="1-methyl-cis-3-ethylcyclohexane"):                           chem = "1-methyl-{\it cis}-3-ethylcyclohexane"
        if (chem=="1-methyl-trans-3-ethylcyclohexane"):                         chem = "1-methyl-{\it trans}-3-ethylcyclohexane"
        if (chem=="1-methyl-cis-3-ethylcyclopentane"):                          chem = "1-methyl-{\it cis}-3-ethylcyclopentane"
        if (chem=="1-methyl-trans-3-ethylcyclopentane"):                        chem = "1-methyl-{\it trans}-3-ethylcyclopentane"
        if (chem=="2-methyl-cis-3-heptene"):                                    chem = "2-methyl-{\it cis}-3-heptene"
        if (chem=="2-methyl-trans-3-heptene"):                                  chem = "2-methyl-{\it trans}-3-heptene"
        if (chem=="3-methyl-cis-3-heptene"):                                    chem = "3-methyl-{\it cis}-3-heptene"
        if (chem=="3-methyl-trans-3-heptene"):                                  chem = "3-methyl-{\it trans}-3-heptene"
        if (chem=="4-methyl-cis-3-heptene"):                                    chem = "4-methyl-{\it cis}-3-heptene"
        if (chem=="4-methyl-trans-3-heptene"):                                  chem = "4-methyl-{\it trans}-3-heptene"
        if (chem=="5-methyl-cis-3-heptene"):                                    chem = "5-methyl-{\it cis}-3-heptene"
        if (chem=="5-methyl-trans-3-heptene"):                                  chem = "5-methyl-{\it trans}-3-heptene"
        if (chem=="6-methyl-cis-3-heptene"):                                    chem = "6-methyl-{\it cis}-3-heptene"
        if (chem=="6-methyl-trans-3-heptene"):                                  chem = "6-methyl-{\it trans}-3-heptene"
        if (chem=="2-methyl-cis-3-hexene"):                                     chem = "2-methyl-{\it cis}-3-hexene"
        if (chem=="2-methyl-trans-3-hexene"):                                   chem = "2-methyl-{\it trans}-3-hexene"
        if (chem=="3-methyl-cis-3-hexene"):                                     chem = "3-methyl-{\it cis}-3-hexene"
        if (chem=="3-methyl-trans-3-hexene"):                                   chem = "3-methyl-{\it trans}-3-hexene"
        if (chem=="1-methyl-cis-3-isopropylcyclopentane"):                      chem = "1-methyl-{\it cis}-3-isopropylcyclopentane"
        if (chem=="1-methyl-trans-3-isopropylcyclopentane"):                    chem = "1-methyl-{\it trans}-3-isopropylcyclopentane"
        if (chem=="1-methyl-cis-3-propylcyclopentane"):                         chem = "1-methyl-{\it cis}-3-propylcyclopentane"
        if (chem=="1-methyl-trans-3-propylcyclopentane"):                       chem = "1-methyl-{\it trans}-3-propylcyclopentane"
        if (chem=="1-methyl-cis-4-ethylcyclohexane"):                           chem = "1-methyl-{\it cis}-4-ethylcyclohexane"
        if (chem=="1-methyl-trans-4-ethylcyclohexane"):                         chem = "1-methyl-{\it trans}-4-ethylcyclohexane"
        if (chem=="methylethyl-sec-butylamine"):                                chem = "methylethyl-{\it sec}-butylamine"
        if (chem=="methylethyl-tert-butylamine"):                               chem = "methylethyl-{\it tert}-butylamine"
        if (chem=="o-chlorobenzoic acid"):                                      chem = "$o$-chlorobenzoic acid"
        if (chem=="cis-2-octene"):                                              chem = "{\it cis}-2-octene"
        if (chem=="trans-2-octene"):                                            chem = "{\it trans}-2-octene"
        if (chem=="cis-3-octene"):                                              chem = "{\it cis}-3-octene"
        if (chem=="trans-3-octene"):                                            chem = "{\it trans}-3-octene"
        if (chem=="cis-4-octene"):                                              chem = "{\it cis}-4-octene"
        if (chem=="trans-4-octene"):                                            chem = "{\it trans}-4-octene"
        if (chem=="cis-1,3-pentadiene"):                                        chem = "{\it cis}-1,3-pentadiene"
        if (chem=="tert-pentyl acetate"):                                       chem = "{\it tert}-pentyl acetate"
        if (chem=="cis-1,1,3,4-tetramethylcyclopentane"):                       chem = "{\it cis}-1,1,3,4-tetramethylcyclopentane"
        if (chem=="trans-1,1,3,4-tetramethylcyclopentane"):                     chem = "{\it trans}-1,1,3,4-tetramethylcyclopentane"
        if (chem=="cis-1,2,2,3-tetramethylcyclopentane"):                       chem = "{\it cis}-1,2,2,3-tetramethylcyclopentane"
        if (chem=="trans-1,2,2,3-tetramethylcyclopentane"):                     chem = "{\it trans}-1,2,2,3-tetramethylcyclopentane"
        if (chem=="3,4,4-trimethyl-cis-2-pentene"):                             chem = "3,4,4-trimethyl-{\it cis}-2-pentene"
        if (chem=="3,4,4-trimethyl-trans-2-pentene"):                           chem = "3,4,4-trimethyl-{\it trans}-2-pentene"
        if (chem=="1,cis-2,cis-3-trimethylcyclohexane"):                        chem = "1,{\it cis}-2,{\it cis}-3-trimethylcyclohexane"
        if (chem=="1,cis-2,trans-3-trimethylcyclohexane"):                      chem = "1,{\it cis}-2,{\it trans}-3-trimethylcyclohexane"
        if (chem=="1,trans-2,cis-3-trimethylcyclohexane"):                      chem = "1,{\it trans}-2,{\it cis}-3-trimethylcyclohexane"
        if (chem=="1,cis-2,cis-4-trimethylcyclohexane"):                        chem = "1,{\it cis}-2,{\it cis}-4-trimethylcyclohexane"
        if (chem=="1,cis-2,trans-4-trimethylcyclohexane"):                      chem = "1,{\it cis}-2,{\it trans}-4-trimethylcyclohexane"
        if (chem=="1,trans-2,cis-4-trimethylcyclohexane"):                      chem = "1,{\it trans}-2,{\it cis}-4-trimethylcyclohexane"
        if (chem=="1,trans-2,trans-4-trimethylcyclohexane"):                    chem = "1,{\it trans}-2,{\it trans}-4-trimethylcyclohexane"
        if (chem=="cis,cis-1,3,5-trimethylcyclohexane"):                        chem = "{\it cis},{\it cis}-1,3,5-trimethylcyclohexane"
        if (chem=="cis,trans-1,3,5-trimethylcyclohexane"):                      chem = "{\it cis},{\it trans}-1,3,5-trimethylcyclohexane"
        if (chem=="1,cis-2,cis-3-trimethylcyclopentane"):                       chem = "1,{\it cis}-2,{\it cis}-3-trimethylcyclopentane"
        if (chem=="1,cis-2,trans-3-trimethylcyclopentane"):                     chem = "1,{\it cis}-2,{\it trans}-3-trimethylcyclopentane"
        if (chem=="1,trans-2,cis-3-trimethylcyclopentane"):                     chem = "1,{\it trans}-2,{\it cis}-3-trimethylcyclopentane"
        if (chem=="1,cis-2,cis-4-trimethylcyclopentane"):                       chem = "1,{\it cis}-2,{\it cis}-4-trimethylcyclopentane"
        if (chem=="1,cis-2,trans-4-trimethylcyclopentane"):                     chem = "1,{\it cis}-2,{\it trans}-4-trimethylcyclopentane"
        if (chem=="1,trans-2,cis-4-trimethylcyclopentane"):                     chem = "1,{\it trans}-2,{\it cis}-4-trimethylcyclopentane"
        if (chem=="1-methyl-4-sec-butylbenzene"):                               chem = "1-methyl-4-{\it sec}-butylbenzene"              
        if (chem=="2-methyl-4-tert-butylheptane"):                              chem = "2-methyl-4-{\it tert}-butylheptane"             
        if (chem=="3-methyl-4-tert-butylheptane"):                              chem = "3-methyl-4-{\it tert}-butylheptane"             
        if (chem=="4-methyl-4-tert-butylheptane"):                              chem = "4-methyl-4-{\it tert}-butylheptane"             

        # skip duplicate entries:
        if (chem=='1,cis-3,cis-5-trimethylcyclohexane'):   continue # same as {\it cis},{\it cis}-1,3,5-trimethylcyclohexane
        if (chem=='1,cis-3,trans-5-trimethylcyclohexane'): continue # same as {\it cis},{\it trans}-1,3,5-trimethylcyclohexane

        # correct casrn if it has been converted to date by xls:
        if (type(casrn) is datetime.datetime):
            oldcasrn = str(casrn)
            casrn = re.sub('([0-9][0-9][0-9][0-9]-[0-9][0-9]-)[0-9]([0-9]).*',
                           r'\1\2', oldcasrn)
            if (casrn[0:2]=='19'):
                casrn = casrn[2:]
            print(f'{i:4} {oldcasrn} was converted to {casrn}')

        # create sum formula in TeX format:
        formula_tex = re.sub('([0-9]+)', r'_{\1}', formula)   # add _{}
        formula_tex = re.sub('{([0-9])}', r'\1', formula_tex) # rm {} if only 1 digit

        # create formula for sorting:
        formula_list = re.findall(r'([A-Z][a-z]?)(\d*)', formula)
        composition = {}
        for elem_num in formula_list:
            elem = elem_num[0]
            num  = elem_num[1]
            if (num==''):
                num = 1
            else:
                num = int(num)
            composition[elem] = num
        formula_sort = '!q1q'
        for element in ['S', 'Cl', 'N', 'O', 'C', 'H']:
            num = composition.get(element,0)
            formula_sort += f' {element}{num:02}'

        # run babel to get inchi from SMILES:
        command = 'echo "%s" | babel -xw -ismi -oinchi' % (smiles)
        output = run(command, shell=True, stdout=PIPE, stderr=PIPE, universal_newlines=True)
        inchi2 = output.stdout.rstrip('\n')
        if (not inchi==inchi2):
            print(f'{i:4} {inchi} is not the same as {inchi2}.')
        # run babel to get inchikey:
        inchikey = get_inchikey_from_smiles(smiles)

        # correction for inchikey:
        if (chem=='tert-octyl mercaptan'):
            chem     = '{\it tert}-octyl mercaptan'
            inchikey = 'QZLAEIZEPJAELS-UHFFFAOYSA-N'

        # ignore stereochemistry:
        if (chem=='l-glutamic acid'):
            casrn    = '617-65-2'
            inchikey = 'WHUUTDBJXJRKMK-UHFFFAOYSA-N'

        # add missing CASRNs:
        if (i==400):  casrn = '51200-81-8'
        if (i==506):  casrn = '2207-04-7'
        if (i==1121): casrn = '3970-59-0'
        if (i==1305): casrn = '994-05-8'
        # correct some CASRNs:
        if (casrn=='6982-25-8'):  casrn = '513-85-9'   # racemat, not threo
        if (casrn=='36653-82-4'): casrn = '124-29-8'   # duplicate CASRN
        if (casrn=='62185-29-9'): casrn = '29772-39-2' # completely wrong CASRN
        if (casrn=='1628-00-2'):  casrn = '628-00-2'
        if (casrn=='66719-5I-5'): casrn = '66719-51-5' # 1, not I
        if (casrn=='66779-42-4'): casrn = ''           # casrn for 3-methyl-5-nonanol unknown
        if (casrn=='996-35-6'):   casrn = '996-35-0'   # dimethylisopropylamine
        if (inchikey=='BUUBMKJJLVCTKU-UHFFFAOYSA-N'): casrn = '62184-90-1'

        # use stereochemistry for inchikey of tartaric acid:
        if (inchikey=='FEWJPZIEWOKRBE-UHFFFAOYSA-N'):
            inchikey = 'FEWJPZIEWOKRBE-JCYAYHJZSA-N'

        # add some missing CASRNs:
        if (inchikey=='HIYTVAULOHUDEC-UHFFFAOYSA-N'): casrn =           '' # 2,3-dimethyl-1-hexanol
        if (inchikey=='QVFKMROLPSPCIX-UHFFFAOYSA-N'): casrn = '66576-57-6' # 3,4-dimethyl-1-hexanol
        if (inchikey=='QABJATQYUASJEM-UHFFFAOYSA-N'): casrn = '60564-76-3' # 4,5-dimethyl-1-hexanol
        if (inchikey=='QYFVEEMPFRRFNN-UHFFFAOYSA-N'): casrn =  '2768-18-5' # 5,5-dimethyl-1-hexanol
        if (inchikey=='YPIJDMPDVZUWJO-UHFFFAOYSA-N'): casrn =           '' # 2,3-dimethyl-2-ethyl-1-butanol
        if (inchikey=='DXDJMJLAABGHOH-UHFFFAOYSA-N'): casrn =           '' # 5,5-dimethyl-2-ethyl-1-hexanol
        if (inchikey=='LVIFBEPHIBJBEU-UHFFFAOYSA-N'): casrn =  '1561-18-8' # 2,5-dimethyl-2-heptanol
        if (inchikey=='CJGWLHDFPXIKQX-UHFFFAOYSA-N'): casrn =           '' # 4,4-dimethyl-2-hexanol
        if (inchikey=='KHSOOQUMPPYGAW-UHFFFAOYSA-N'): casrn =           '' # 4,5-dimethyl-2-hexanol
        if (inchikey=='OQEVAISXHCRQGF-FNORWQNLSA-N'): casrn = '19550-83-5' # 4,4-dimethyl-trans-2-hexene
        if (inchikey=='OQEVAISXHCRQGF-ALCCZGGFSA-N'): casrn =           '' # 4,4-dimethyl-cis  -2-hexene       (wrong CASRN deleted)
        if (inchikey=='UMUGNPFWQJAOJI-HLTSFMKQSA-N'): casrn =           '' # 1,cis  -2-dimethyl-cis  -3-ethylcyclopentane
        if (inchikey=='UMUGNPFWQJAOJI-IWSPIJDZSA-N'): casrn =           '' # 1,cis  -2-dimethyl-trans-3-ethylcyclopentane
        if (inchikey=='UMUGNPFWQJAOJI-VGMNWLOBSA-N'): casrn =           '' # 1,trans-2-dimethyl-cis  -3-ethylcyclopentane
        if (inchikey=='UMUGNPFWQJAOJI-HRDYMLBCSA-N'): casrn =           '' # 1,trans-2-dimethyl-trans-3-ethylcyclopentane
        if (inchikey=='UAVZZCGGBCLHIP-UHFFFAOYSA-N'): casrn =           '' # 4,5-dimethyl-3-hexanol
        if (inchikey=='VMCXXGFUCWAIIN-YIZRAAEISA-N'): casrn =           '' # 1,cis  -3-dimethyl-cis  -4-ethylcyclopentane
        if (inchikey=='VMCXXGFUCWAIIN-DJLDLDEBSA-N'): casrn =           '' # 1,cis  -3-dimethyl-trans-4-ethylcyclopentane
        if (inchikey=='VMCXXGFUCWAIIN-CIUDSAMLSA-N'): casrn =           '' # 1,trans-3-dimethyl-cis  -4-ethylcyclopentane
        if (inchikey=='VMCXXGFUCWAIIN-XHNCKOQMSA-N'): casrn =           '' # 1,trans-3-dimethyl-trans-4-ethylcyclopentane
        if (inchikey=='DWEXNJRCPKKJBQ-UHFFFAOYSA-N'): casrn =           '' # 2,2-dimethyl-4-propylheptane
        if (inchikey=='RLGDVTUGYZAYIX-UHFFFAOYSA-N'): casrn = '66576-32-7' # 4-ethyl-1-hexanol
        if (inchikey=='ALAPJJMGVQNJIU-UHFFFAOYSA-N'): casrn =           '' # 4-ethyl-2-hexanol
        if (inchikey=='BUJFTKPQXSIZFX-UHFFFAOYSA-N'): casrn =  '2978-64-5' # methyl-1,1-dimethylpropylamine
        if (inchikey=='MMQDVLWWGWJSFS-UHFFFAOYSA-N'): casrn = '19780-39-3' # 3-ethyl-2-heptanol
        if (inchikey=='ALPFTHGDLMTYRM-UHFFFAOYSA-N'): casrn =           '' # 3-methyl-2-ethyl-1-pentanol
        if (inchikey=='YNZQSDTXBBBFQH-UHFFFAOYSA-N'): casrn =           '' # 1-methyl-3-ethyl-1-hexanol
        if (inchikey=='VWQDMMBTEFDWNM-UHFFFAOYSA-N'): casrn =           '' # 2-methyl-3-ethyl-1-pentanol
        if (inchikey=='XYUCPYMMHKJJSS-UHFFFAOYSA-N'): casrn = '10524-71-7' # 3-methyl-3-ethyl-1-pentanol
        if (inchikey=='RWIFVESHBHTZEM-UHFFFAOYSA-N'): casrn = '38514-13-5' # 4-methyl-3-ethyl-1-pentanol
        if (inchikey=='FGJMKJVRSBVZPS-UHFFFAOYSA-N'): casrn =           '' # 5-methyl-3-ethyl-3-heptanol
        if (inchikey=='QMHNYDJBWOPGHE-UHFFFAOYSA-N'): casrn = '66577-03-5' # 3-methyl-4-ethyl thiophene
        if (inchikey=='NZOYEHPXDWJOCD-UHFFFAOYSA-N'): casrn =           '' # 3-methyl-5-ethylthiophene
        if (inchikey=='KJWHJDGMOQJLGF-UHFFFAOYSA-N'): casrn =  '3698-89-3' # methyl dodecyl sulfide
        if (inchikey=='FNWCWFQEEKFGHB-UHFFFAOYSA-N'): casrn =  '3178-30-1' # 2,2,4,4-tetramethyl-3-ethylpentane
        if (inchikey=='CXCBKSYSKZEEJB-HTQZYQBOSA-N'): casrn =           '' # 1,1,cis-2,trans-3-tetramethylcyclopentane
        if (inchikey=='VIDOPANCAUPXNH-UHFFFAOYSA-N'): casrn = '42205-08-3' # 1,2,3-triethylbenzene
        if (inchikey=='LFRFDOWJBYDJGD-UHFFFAOYSA-N'): casrn =           '' # 2,3,3-trimethyl-1-pentanol
        if (inchikey=='OIPMJJVCOSACDS-UHFFFAOYSA-N'): casrn =           '' # 3,3,4-trimethyl-1-pentanol
        if (inchikey=='LTDMOZOPTDNTLG-UHFFFAOYSA-N'): casrn =           '' # 3,3,6-trimethyl-4-heptanol

        # correction for inchikey:
        if (chem=='nitroglycerine'):
            inchikey = 'SNIOPGDIGTZGOP-UHFFFAOYSA-N'

        # neutral acid, not anion:
        if (chem=='ethylacetoacetate'):
            inchikey = 'XYIBRDXRRQCHLP-UHFFFAOYSA-N'

        # skip 1-methyl-2-ethyl-1-hexanol:
        if (chem=='1-methyl-2-ethyl-1-hexanol'):
            print(i, 'Skipping 1-methyl-2-ethyl-1-hexanol (same as 3-ethyl-2-heptanol)')
            continue

        # skip vinylcyclohexene:
        if (chem=='vinylcyclohexene'):
            print(i, 'Skipping vinylcyclohexene because it is unclear if this is')
            print('1-vinylcyclohexene or 4-vinylcyclohexene.')
            continue

        if (chem=='1,1,cis-2,cis-3-tetramethylcyclopentane'):
            chem     = '{\it cis}-1,1,2,3-tetramethylcyclopentane'
            inchikey = 'CXCBKSYSKZEEJB-YUMQZZPRSA-N' # also CXCBKSYSKZEEJB-JGVFFNPUSA-N ?
        if (chem=='1,1,cis-2,trans-3-tetramethylcyclopentane'):
            chem     = '{\it trans}-1,1,2,3-tetramethylcyclopentane'
            casrn    = '62016-70-0'
            inchikey = 'CXCBKSYSKZEEJB-SFYZADRCSA-N'

        # 1,2,3,4-tetramethylcyclopentane:
        if (chem=='1,cis-2,cis-3,cis-4-tetramethylcyclopentane'):
            chem = '1,{\it cis}-2,{\it cis}-3,{\it cis}-4-tetramethylcyclopentane'
        if (chem=='1,cis-2,cis-3,trans-4-tetramethylcyclopentane'):
            chem = '1,{\it cis}-2,{\it cis}-3,{\it trans}-4-tetramethylcyclopentane'
            inchikey = 'INYXDKODFMWKER-BGZDPUMWSA-N'
        if (chem=='1,cis-2,trans-3,cis-4-tetramethylcyclopentane'):
            chem = '1,{\it cis}-2,{\it trans}-3,{\it cis}-4-tetramethylcyclopentane'
        if (chem=='1,cis-2,trans-3,trans-4-tetramethylcyclopentane'):
            print(i, 'Skipping 1,cis-2,trans-3,trans-4-tetramethylcyclopentane')
            print('because inchikey is unknown and casrn is questionable.')
            continue
        if (chem=='1,trans-2,cis-3,trans-4-tetramethylcyclopentane'):
            chem = '1,{\it trans}-2,{\it cis}-3,{\it trans}-4-tetramethylcyclopentane'
            inchikey = 'INYXDKODFMWKER-HXFLIBJXSA-N'
        if (chem=='1,trans-2,trans-3,cis-4-tetramethylcyclopentane'):
            chem = '1,{\it trans}-2,{\it trans}-3,{\it cis}-4-tetramethylcyclopentane'
            
        # 1,1,2,4-tetramethylcyclopentane:
        if (chem=='1,1,cis-2,cis-4-tetramethylcyclopentane'):
            chem     = '{\it cis}-1,1,2,4-tetramethylcyclopentane'
            casrn    = '62016-71-1'
            inchikey = 'AVBGIJNNMIBMQG-SFYZADRCSA-N'
        if (chem=='1,1,cis-2,trans-4-tetramethylcyclopentane'):
            chem     = '{\it trans}-1,1,2,4-tetramethylcyclopentane'
            casrn    = ''
            inchikey = 'AVBGIJNNMIBMQG-YUMQZZPRSA-N' # also AVBGIJNNMIBMQG-HTQZYQBOSA-N ?

        # correct 1,1,3,4-tetramethylcyclopentane data:
        if (chem=='1,1,cis-3,cis-4-tetramethylcyclopentane'):
            chem     = '{\it cis}-1,1,3,4-tetramethylcyclopentane'
            casrn    = '53907-60-1'
            inchikey = 'OWHFMVURUNNXMJ-OCAPTIKFSA-N'
        if (chem=='1,1,cis-3,trans-4-tetramethylcyclopentane'):
            chem     = '{\it trans}-1,1,3,4-tetramethylcyclopentane'
            casrn    = '20309-77-7'
            inchikey = 'OWHFMVURUNNXMJ-HTQZYQBOSA-N'

        # correct 1,2,2,3-tetramethylcyclopentane (also called 1,1,2,5-) data:
        if (chem=='1,2,2,cis-3-tetramethylcyclopentane'):
            chem     = '{\it cis}-1,2,2,3-tetramethylcyclopentane'
            casrn    = '18938-68-6'
        if (chem=='1,2,2,trans-3-tetramethylcyclopentane'):
            chem     = '{\it trans}-1,2,2,3-tetramethylcyclopentane'
            casrn    = ''
            inchikey = 'DHLYDHNCPUAVHP-YUMQZZPRSA-N'

        # get casrn, if species is already in henry.f90:
        casrn2 = get_casrn_from_henry_f90()
        if (casrn2=='2074-87-5'): # wrong SMILES: (CN)2, not CN
            casrn2   = '460-19-5'
            inchikey = 'JMANVNJQNLATNU-UHFFFAOYSA-N'
        if (casrn2=='138-86-3'): # wrong SMILES: d-limonene, not limonene
            casrn2   = '5989-27-5'
            inchikey = 'XMGQYMWWDOXHJM-SNVBAGLBSA-N'
        if (casrn=='57-55-6'): # wrong SMILES, use CASRN from table
            casrn2='57-55-6'
            inchikey = 'DNIAPMSPPWPWGF-UHFFFAOYSA-N'
        if (casrn2==''):
            if (casrn=='nan'):
                casrn = ''
                print(f'{i:4} No CASRN for {inchikey} {chem}.')
            print(f'{formula_sort} {formula:15}!  CALL Species("{chem}", "{formula_tex}", "","{casrn}", "{inchikey}")', file=f_f90)
            # print(f'  CALL Species("{chem}", "{formula_tex}", "", &', file=f_f90)
            # print(f'    "{casrn}", "{inchikey}")', file=f_f90)
        else:
            if (casrn==casrn2):
                # get inchikey from henry.f90:
                inchikey2 = get_inchikey_from_henry_f90()
                if (not inchikey==inchikey2):
                    print(f'{i:4} Inchikey from table ({inchikey}) is different ' +
                          f'from that in henry.f90 ({inchikey2}).')
            else:
                print(f'{i:4} CASRN from table ({casrn}) is different from that ' +
                      f'calculated from SMILES ({casrn2}).')

        print(f'{i:4} {H_yaws:8} {H:8} {temp:5} "{inchikey}" "{chem}"', file=f_dat)
        namedict.append(f'        {chemstr:50}: "{inchikey}"')
        
    print(',\n'.join(namedict) + '}', file=f_dict)
    f_dat.close()
    f_dict.close()
    f_f90.close()

    # create f90 file that define the MCM species with "CALL Species":
    os.system('sort ref3579-raw.f90 > ref3579.f90')
    os.remove('ref3579-raw.f90')
