#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pandas as pd
import re
import os, sys
from subprocess import PIPE, run
import urllib.request

def grep_i(searchstring, filename):
    # suffix _i indicates grep option '-i' for case insensitive
    results = []
    with open(filename) as f:
        for line in f:
            if (searchstring in line):
                results.append(line.strip())
    return results

def get_casrn_from_henry_f90():
    if (not inchikey): return ''
    result = grep_i('"'+inchikey+'"', '../henry.f90')
    if (result):
        return re.sub('.*"([^"]*)", *"[-A-Z]+".*', r'\1', result[0], flags=re.IGNORECASE)
    return ''

def get_inchikey_from_henry_f90():
    result = grep_i('"'+casrn+'"', '../henry.f90')
    if (result):
        return re.sub('.*, "([^"]*)".*', r'\1', result[0])
    else:
        return ''

def get_inchikey_from_smiles(smiles):
    command = 'echo "%s" | babel -xw -ismi -oinchikey' % (smiles)
    output = run(command, shell=True, stdout=PIPE, stderr=PIPE, universal_newlines=True)
    inchikey = output.stdout.rstrip('\n')
    return inchikey

def create_CALL_Species(species, casrn, inchikey):
    print(f'NEW: {N:>4} {casrn:>14} {inchikey} {species}')

if __name__ == '__main__':

    # https://pandas.pydata.org/docs/reference/api/pandas.read_csv.html
    df = pd.read_csv('ref3586.csv', skipinitialspace=True)

    f_dat  = open('../input/ref3586.dat', 'w')

    for index, row in df.iterrows():

        #if (index>20): break #qqq only a few lines for testing
        N        = row['N']
        casrn    = row['CAS'].lstrip("0") # remove leading zeros
        chem     = row['Compound']
        smiles   = row['SMILES']
        Exp_Est  = row['Exp_Est']
        Eq1_Eq41 = row['Eq1_Eq41']

        info = ''
        if ('^' in N): info = 'val'  # validation set
        if ('*' in N): info = 'test' # test set
        N = int(re.sub('[*^]', '', N))
        if (N>530): info = 'V' # type V

        # run babel to get inchikey:
        inchikey = get_inchikey_from_smiles(smiles)

        # skip mixtures because they are not pure substances:
        if (chem=="XYLENES (ISOMER MIXTURE)"):     continue #  394
        if (chem=="AROCLOR 1260"):                 continue #  415
        if (chem=="AROCLOR 1254"):                 continue #  416
        if (chem=="AROCLOR 1221"):                 continue #  417
        if (chem=="AROCLOR 1248"):                 continue #  418
        if (chem=="AROCLOR 1242"):                 continue #  501
        if (chem=="AROCLOR 1232"):                 continue # 1166
        if (chem=="NONYLPHENOL (ISOMER MIXTURE)"): continue # 1182
        if (chem=="HEXACHLORODIBENZO-P-DIOXIN"):   continue # 1208
        if (chem=="PENTACHLORODIBENZO-P-DIOXIN"):  continue # 1213
        if (chem=="HEXABROMOBIPHENYL"):            continue # 1214
        if (chem=="HEPTACHLORODIBENZO-P-DIOXIN"):  continue # 1216
        if (chem=="DIISOPROPYLNAPHTHALENE"):       continue # 1220
        if (chem=="IMAZAMETHABENZ (ISOMER MIX)"):  continue # 1293
        if (chem=="N-NONYLPHENOL"):                continue # 1301

        # skip because name and SMILES don't match:
        if (chem=="BENFURESATE"):                  continue # 1272 should be QGQSRQPXXMTJCM-UHFFFAOYSA-N, not YEIIYOHLGJZIJE-UHFFFAOYSA-N

        # skip because it is unclear what isomer is used:
        if (chem=="ISOPROPYLBIPHENYL"):            continue # 1186 (casrn 25640-78-2 is for all isomers)

        # skip because name and casrn don't match:
        #   casrn 29460-93-3 is for sec-butylpyrazine
        #   casrn 29460-92-2 is for isobutylpyrazine
        if (chem=="ISOBUTYLPYRAZINE"):             continue #  435

        # wrong casrn:
        if (N==   1): casrn =  "82944-59-0" #      2-52-9 3-pentyl nitrate
        if (N== 531): casrn =   "1195-92-2" #      3-73-3 limonene oxide
        if (N==1183): casrn =     "99-87-6" #  25155-15-1 p-i-propyltoluene
        if (N==1185): casrn =    "622-96-8" #  25550-14-5 methyl ethyl benzene (para- according to SMILES)
        if (N==1189): casrn =     "78-99-9" #  26638-19-7 dichloropropane (1,1- according to SMILES)
        if (N==1191): casrn =    "693-02-7" #  26856-30-4 hexyne (1- according to SMILES)
        if (N==1192): casrn =   "1653-40-3" #  26952-21-6 isooctyl alcohol (6-methylheptanol according to SMILES)
        if (N==1197): casrn =  "10453-86-8" #  28434-01-7 bioresmethrin (now using the casrn w/o stereochemistry)

        # if species has more than 1 casrn, always use the same:
        if (casrn=="11039-70-6"): casrn = "72402-00-7" # 1165
        if (casrn=="36653-82-4"): casrn =   "124-29-8" # 1215
        if (casrn=="72391-46-9"): casrn = "84332-86-5" # 1280 chlozolinate
        if (casrn=="79127-80-3"): casrn = "72490-01-8" # 1292

        # stereochemistry is missing in inchikeys generated from SMILES:
        if (N== 293): inchikey = "BATOPAZDIZEVQF-MQQKCMAXSA-N" #    142-83-6
        if (N==1146): inchikey = "BIWJNBZANLAXMG-KMMBHOGFSA-N" #   5103-71-9
        if (N==1147): inchikey = "BIWJNBZANLAXMG-SVSLFMFOSA-N" #   5103-74-2
        if (N==  28): inchikey = "DFBKLUNHFCTMDC-GKRDHZSOSA-N" #     72-20-8
        if (N==  13): inchikey = "DFBKLUNHFCTMDC-NLUYNBKHSA-N" #     60-57-1
        if (N== 786): inchikey = "FQDIANVAWVHZIR-OWOJBTEDSA-N" #    110-57-6
        if (N==1080): inchikey = "FQDIANVAWVHZIR-UPHRSURJSA-N" #   1476-11-5
        if (N== 948): inchikey = "GIKLTQKNOXNBNY-OWOJBTEDSA-N" #    541-25-3
        if (N==1114): inchikey = "IAIHUHQCLTYTSF-OYNCUSHFSA-N" #   2217-02-9
        if (N== 353): inchikey = "IAQRGUVFOMOMEM-ARJAWSKDSA-N" #    590-18-1
        if (N== 362): inchikey = "IAQRGUVFOMOMEM-ONEGZZNKSA-N" #    624-64-6
        if (N==1268): inchikey = "IQVNEKKDSLOHHK-FNCQTZNRSA-N" #  67485-29-4
        if (N==1315): inchikey = "JBDHZKLJNAIJNC-LLVKDONJSA-N" # 105512-06-9
        if (N== 311): inchikey = "JLYXXMFPNIAWKQ-CDRYSYESSA-N" #    319-85-7
        if (N==  10): inchikey = "JLYXXMFPNIAWKQ-GNIYUCBRSA-N" #     58-89-9
        if (N== 903): inchikey = "JLYXXMFPNIAWKQ-GPIVLXJGSA-N" #    319-86-8
        if (N== 710): inchikey = "KBEBGUQPQBELIU-CMDGGOBGSA-N" #    103-36-6
        if (N==1155): inchikey = "KVZJLSYJROEPSQ-HTQZYQBOSA-N" #   6876-23-9
        if (N==1111): inchikey = "KVZJLSYJROEPSQ-OCAPTIKFSA-N" #   2207-01-4
        if (N==1178): inchikey = "KZAUOCCYDRDERY-UITAMQMPSA-N" #  23135-22-0
        if (N==1026): inchikey = "LDCRTTXIJACKKU-ARJAWSKDSA-N" #    624-48-6
        if (N== 919): inchikey = "LDHQCZJRKDOVOX-IHWYPQMZSA-N" #    503-64-0
        if (N== 270): inchikey = "MLUCVPSAIODCQM-NSCUHMNNSA-N" #    123-73-9
        if (N==1145): inchikey = "NKKMVIVFRUYPLQ-NSCUHMNNSA-N" #   4786-20-3
        if (N==1263): inchikey = "NYPJDWWKZLNGGM-ZEQRLZLVSA-N" #  66230-04-4
        if (N==1296): inchikey = "OMFRMAHOUUJSGP-UNOMPAQXSA-N" #  82657-04-3
        if (N==1169): inchikey = "OTTZHAVKAVGASB-HWKANZROSA-N" #  14686-13-6
        if (N==1150): inchikey = "OWBTYPJTUOEWEK-ZXZARUISSA-N" #   5341-95-7
        if (N==1244): inchikey = "OWZREIFADZCYQD-NSHGMRRFSA-N" #  52918-63-5
        if (N== 708): inchikey = "PJANXHGTPQOBST-VAWYXSNFSA-N" #    103-30-0
        if (N==1277): inchikey = "PQKBPHSEKWERTG-LLVKDONJSA-N" #  71283-80-2
        if (N==1170): inchikey = "PSQYTAPXSHCGMF-BQYQJAHWSA-N" #     79-77-6
        if (N== 310): inchikey = "QBYJBZPUGVGKQQ-SJJAEHHWSA-N" #    309-00-2
        if (N== 822): inchikey = "QGLZXHRNAYXIBU-WEVVVXLNSA-N" #    116-06-3
        if (N==1049): inchikey = "QMMOXUPEWRXHJS-HWKANZROSA-N" #    646-04-8
        if (N==1036): inchikey = "QMMOXUPEWRXHJS-HYXAFXHYSA-N" #    627-20-3
        if (N==1316): inchikey = "QNBTYORWCCMPQP-JXAWBTAJSA-N" # 110488-70-5
        if (N==1112): inchikey = "QRMPKOFEUHIBNM-ZKCHVHJHSA-N" #   2207-04-7
        if (N== 387): inchikey = "RDYMFSUJUZBWLH-AMHWMVONSA-N" #    959-98-8
        if (N== 443): inchikey = "RDYMFSUJUZBWLH-MDBBVBRHSA-N" #  33213-65-9
        if (N== 839): inchikey = "ROVGZAWFACYCSP-CMDGGOBGSA-N" #    121-21-1
        if (N==1309): inchikey = "RVKCCVTVZORVGD-QXMHVHEDSA-N" #  95266-40-3
        if (N== 288): inchikey = "VEENJGZXVHKXNB-VOTSOKGWSA-N" #    141-66-2
        if (N== 840): inchikey = "VJFUPGQZSXIULQ-VKTMSVCMSA-N" #    121-29-9
        if (N==1152): inchikey = "XMGQYMWWDOXHJM-SNVBAGLBSA-N" #   5989-27-5
        if (N==1266): inchikey = "YWSCPYYRJXKUDB-KAKFPZCNSA-N" #  66841-25-6
        if (N== 534): inchikey = "ZPUCINDJVBIVPJ-PFSRBDOWSA-N" #     50-36-2
        if (N==1276): inchikey = "ZQEIXNIJLIKNTD-GFCCVEGCSA-N" #  70630-17-0

        # stereochemistry is missing in inchikeys generated from SMILES for new species:
        if (casrn==   "561-27-3"): inchikey = "GVGLGOZIDCSQPN-PVHGPHFFSA-N" # "HEROIN"
        if (casrn==   "106-25-2"): inchikey = "GLZPCOQZEFWAFX-YFHOEESVSA-N" # "2,6-OCTADIEN-1-OL, 3,7-DIMETHYL-, (Z)-"
        if (casrn==   "105-76-0"): inchikey = "JBSLOWBPDRZSMB-FPLPWBNLSA-N" # "DIBUTYL MALEATE"
        if (casrn==  "1435-60-5"): inchikey = "NRJPVIOTANUINF-YPKPFQOOSA-N" # "4-(4-NITROPHENYLAZO)PHENOL"
        if (casrn=="133408-50-1"): inchikey = "HIIRDDUVRXCDBN-SDXDJHTJSA-N" # "SSF-126"
        if (casrn==  "3025-52-3"): inchikey = "LVQIWDUSUJTZJF-ISLYRVAYSA-N" # "DIS. A. 5"
        if (casrn==  "2872-52-8"): inchikey = "FOQABOMYTOFLPZ-ZCXUNETKSA-N" # "DISPERSE RED 1"
        if (casrn== "88485-37-4"): inchikey = "UKSLKNUCVPZQCQ-GZTJUZNOSA-N" # "FLUXOFENIM"
        if (casrn== "63782-90-1"): inchikey = "IKVXBIIHQGXQRQ-CYBMUJFWSA-N" # "FLAMPROP-M-ISOPROPYL"
        if (casrn==   "590-27-2"): inchikey = "CVOGMKGEVNGRSK-OWOJBTEDSA-N" # "1,2-DIIODOETHENE (TRANS)"
        if (casrn== "83130-01-2"): inchikey = "GMAUQNJOSOMMHI-JXAWBTAJSA-N" # "ALANYCARB"

        # wrong casrn and stereochemistry is missing in inchikeys generated from SMILES:
        if (N==1318):
            casrn = "134098-61-6" # 111812-58-9 fenpyroximate
            inchikey = "YYJNOYZRYGDPNH-MFKUBSTISA-N"

        # inchikey generated from SMILES seems not correct, but name and casrn are okay:
        if (casrn==   "543-29-3"): inchikey = "LNNXFUZKZLXPOF-UHFFFAOYSA-N" #  337 SMILES produces another isomer: DYONNFFVDNILGI-UHFFFAOYSA-N
        if (casrn== "67028-18-6"): inchikey = "SKGXYFVQZVPEFP-UHFFFAOYSA-N" #  511 SMILES produces another isomer: BXKLTNKYLCZOHF-UHFFFAOYSA-N
        if (casrn== "68194-06-9"): inchikey = "BWWVXHRLMPBDCK-UHFFFAOYSA-N" #  513 SMILES produces another isomer: SUOAMBOBSWRMNQ-UHFFFAOYSA-N
        if (casrn=="113136-77-9"): inchikey = "GLWWLNJJJCTFMZ-UHFFFAOYSA-N" #  527
        if (casrn==  "2008-39-1"): inchikey = "IUQJDHJVPLLKFL-UHFFFAOYSA-N" # 1103
        if (casrn==  "2439-10-3"): inchikey = "YIKWKLYQRFRGPM-UHFFFAOYSA-N" # 1123
        if (casrn==  "5409-83-6"): inchikey = "IVVRJIDVYSPKFZ-UHFFFAOYSA-N" # 1151 SMILES produces another isomer: DMOBGFAKTSXOHG-UHFFFAOYSA-N
        if (casrn== "25168-26-7"): inchikey = "BBPLSOGERZQYQC-UHFFFAOYSA-N" # 1184
        if (casrn== "38964-22-6"): inchikey = "WMWJCKBJUQDYLM-UHFFFAOYSA-N" # 1221 SMILES is for 1,8-dichlorodibenzo-p-dioxin
        if (casrn=="104653-34-1"): inchikey = "VSVAQRUUFVBBFS-UHFFFAOYSA-N" # 1314 SMILES produces another isomer: JHELOZJAKXYVBE-UHFFFAOYSA-N

        # casrn is for isomer mix:
        if (casrn== "56803-37-3"): #  506
            casrn    = "981-40-8"
            inchikey = "ULGAVXUJJBOWOD-UHFFFAOYSA-N"

        # use casrn for (-)-Borneol:
        if (casrn== "507-70-0"): # 924
            casrn    = "464-45-9"
            inchikey = "DTGKSKDOIYIVQL-QXFUBDJGSA-N"

        casrn_check    = ''
        inchikey_check = ''
        # get inchikey from henry.f90:
        inchikey2 = get_inchikey_from_henry_f90()
        casrn2    = get_casrn_from_henry_f90()
        if (casrn2):
            if (not casrn==casrn2):
                casrn_check = casrn2
        else:
            casrn_check = 'inchikey not in henry.f90'
        if (inchikey2):
            if (not inchikey==inchikey2):
                inchikey_check = inchikey2
        else:
            inchikey_check = 'casrn not in henry.f90'

        if (casrn_check or inchikey_check):
            if (casrn2 or inchikey2):
                print(f'{N:>4}: {casrn:>11} {inchikey} {casrn_check:27} {inchikey_check:27}')
            else:
                create_CALL_Species(chem, casrn, inchikey)

        quoted_info = '"' + info + '"'
        quoted_casrn = '"' + casrn + '"'
        print(f'{N:>4} {Exp_Est:10} {Eq1_Eq41:10} {quoted_info:6} "{inchikey}" {quoted_casrn:>14} "{chem}"', file=f_dat)

    f_dat.close()
