#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pandas as pd
import re
import os, sys
from subprocess import PIPE, run
#import urllib.request

def grep_i(searchstring, filename):
    # suffix _i indicates grep option '-i' for case insensitive
    results = []
    with open(filename) as f:
        for line in f:
            if (searchstring in line):
                results.append(line.strip())
    return results

def find_inchikey_in_henry_f90(inchikey):
    result = grep_i(inchikey, '../henry.f90')
    if (result):
        return inchikey
    else:
        return ''

def get_png_from_smiles(name, smiles):
    command = f'echo "{smiles}" | obabel -ismi -opng -O ref3765_{name}.png'
    run(command, shell=True, stdout=PIPE, stderr=PIPE)

def get_inchikey_from_smiles(smiles):
    command = f'echo "{smiles}" | obabel -ismi -oinchikey'
    output = run(command, shell=True, stdout=PIPE, stderr=PIPE, universal_newlines=True)
    inchikey = output.stdout.rstrip('\n')
    return inchikey

def get_texformula_from_smiles(smiles):
    command = f'echo "{smiles}" | obabel -ismi -omolreport'
    output = run(command, shell=True, stdout=PIPE, stderr=PIPE, universal_newlines=True)
    result = re.search('FORMULA: ([A-z0-9]+)', output.stdout)
    if (result):
        # all numbers as subscripts in brackets:
        texformula = re.sub('([A-z]+)([0-9]+)', r'\1_{\2}', result.group(1))
        # remove brackets for 1-digit numbers:
        texformula = re.sub('{([0-9])}', r'\1', texformula)
        return texformula
    else:
        return ''

def create_CALL_Species(species, casrn, inchikey):
    print(f'NEW: {N:>4} {casrn:>14} {inchikey} {species}')

if __name__ == '__main__':

    # https://pandas.pydata.org/docs/reference/api/pandas.read_csv.html
    df = pd.read_csv('ref3765.csv', skipinitialspace=True, dtype='str')
    #print(df.shape)   
    #print(df.columns)
  
    f_Species = open('ref3765_CALL_Species.f90', 'w')
    f_CalcH   = open('ref3765_CALL_CalcH.f90',   'w')

    for index, row in df.iterrows():
        #if (index>10): break #qqq only a few lines for testing
        name     = row['NAME']       
        smiles   = row['SMILES']     
        inchikey = row['InChIKey']   
        Hscp     = row['Hscp']       
        B        = row['B']
        print(name)
        #print(name, smiles, inchikey, Hscp, B)
        texformula = get_texformula_from_smiles(smiles)
        # run babel to get inchikey:
        inchikey3 = get_inchikey_from_smiles(smiles)
        if (not inchikey==inchikey3):
            print(f'babel creates different inchikey: {inchikey}, {inchikey3}')
        quoted_name = '"' + name + '",'
        quoted_Hscp = Hscp + ','
        print(f'    CALL CalcH({quoted_name:24} "{inchikey}", {quoted_Hscp:10} {B})',
              file=f_CalcH)
            
        # find inchikey in henry.f90:
        if (not find_inchikey_in_henry_f90(inchikey)):
            # create image of structural formula:
            get_png_from_smiles(name, smiles)
            trivialname = ''
            casrn = ''
            # manually add some proper names and casrn:
            if (name=='ME1TRIOL'):
                trivialname = name
                name = '2-pentene-2,4,5-triol'
            if (name=='MeBuTETROL'):
                trivialname = name
                name = '2-Methylbutane-1,2,3,4-tetrol'
                casrn = '42933-13-1'
            if (name=='C7H15ONO2'):
                trivialname = name
                name = '2-heptyl nitrate'
            if (name=='C7H15O2H'):
                trivialname = name
                name = '1-methylhexyl hydroperoxide'
                casrn = '762-46-9'
            if (name=='C8H17ONO2'):
                trivialname = name
                name = '2-octyl nitrate'
            if (name=='C8H17O2H'):
                trivialname = name
                name = '1-methylheptl hydroperoxide'
            if (name=='C82CO5OH'):
                trivialname = name
                name = '5-hydroxy-2-octanone'
            # create f90 code for CALL Species:
            quoted_casrn = '"' + casrn + '",'
            quoted_texformula = '"' + texformula + '",'
            print(f'  CALL Species({quoted_texformula:35} {quoted_casrn:>15} "{inchikey}", "{name}", "{trivialname}") ! SMILES: {smiles}', file=f_Species)

    f_Species.close()
    f_CalcH.close()
