#!/usr/bin/env python3
# -*- coding: utf-8 -*- Time-stamp: <2023-05-03 13:32:17 sander>

import numpy as np
import matplotlib.pyplot as plt
from henry_util import CtoK, rhoH2O, psat, MCl, atm, Rgas, p0

##############################################################################

def make_Khyd_plot():
    plt.figure()
    plt.xlabel('T')
    plt.ylabel('K_hyd')

    # ------------------------------------------------------------------------

    # CCl4 method:
    
    # ref3925, Fig. 1,
    # temp from original ref3937 tab. 46, Khyd digitized with engauge:
    ref3925_fig1_CCl4_temp = np.array([ 0,      13.4,   28.6,  38.6,   51.2,    57.5 ])
    ref3925_fig1_CCl4_Khyd = np.array([ 0.7472, 1.4976, 2.557, 3.3157, 4.3076, 4.673 ])
    # convert Cl2 from g to mol:
    ref3925_fig1_CCl4_Khyd = ref3925_fig1_CCl4_Khyd / (1000*MCl*2)**2
    plt.plot(ref3925_fig1_CCl4_temp, ref3925_fig1_CCl4_Khyd, linestyle='', marker='o')
    
    # Tab. 46 (not used):
    tab46_temp = np.array([ 13.4,  13.4,  13.4,   28.6,  28.6,   38.6,  38.6,   51.2,  51.2,   57.5  ])
    tab46_pres = np.array([ 168.5, 37.94, 10.25,  142.9, 26.01,  126.1, 22.64,  118.0, 29.1,   114.0 ])
    tab46_03 = np.array([ 6.693, 1.493, 0.4025, 4.75,  0.8594, 3.971, 0.7051, 3.535, 0.864,  3.31  ])
    tab46_04 = np.array([ 6.642, 1.482, 0.3995, 4.689, 0.8482, 3.906, 0.6935, 3.459, 0.8454, 3.234 ])
    tab46_05 = np.array([ 25.18, 25.41, 25.46,  30.10, 30.28,  31.76, 32.11,  33.38, 33.68,  34.44 ])
    tab46_06 = np.array([ 25.35, 25.60, 25.64,  30.48, 30.66,  32.33, 32.68,  34.1,  34.4,   35.23 ])
    # Tabs. 47-51, k from p. 717:
    p717_k   = np.array([ 0.000156, 0.000303, 0.000519, 0.000664, 0.000867, 0.00094 ])
    plt.plot(ref3925_fig1_CCl4_temp, p717_k, linestyle='', marker='+')

    # ------------------------------------------------------------------------

    # HCl method:
    
    # ref3925, Fig. 1,
    # temp from original ref3937 tab. 52, Khyd digitized with engauge:
    ref3925_fig1_HCl_temp  = np.array([ 15., 25.0, 39.1, 53.6, 67.6, 83.4 ])
    ref3925_fig1_HCl_Khyd  = np.array([ 1.5678, 2.2142,  3.4, 4.5084, 5.1733, 5.463  ])
    # convert Cl2 from g to mol:
    ref3925_fig1_HCl_Khyd = ref3925_fig1_HCl_Khyd / (1000*MCl*2)**2
    plt.plot(ref3925_fig1_HCl_temp, ref3925_fig1_HCl_Khyd, linestyle='', color='red', marker='o')

    # Tab. 52:
    # 1                         = temperature
    # 2                         = pressure
    # 3 Величинь Поглощеня      = amount of absorbed chlorine in 1/5 N HCl
    # 4 Соотввтствующня Имъ     = corresponding active mass of chlorine
    # 9 Степень гидролива хлора = degree of chlorine hydrolysis
    # 10 Найденная              = found hydrolysis constant 
    tab52_01  = np.array([ 15.0,     25.0,     39.1,     53.6,        67.6,        83.4        ])
    tab52_02  = np.array([ 749.,     768.,     763.,     758.,        753.,        753.        ])
    tab52_03  = np.array([ 85.4,     61.3,     37.26,    23.16,       14.45,       7.158       ])
    tab52_04  = np.array([ 84.72,    60.6,     36.64,    22.65,       14.09,       6.979       ])
    #tab52_05 = np.array([ 87.1,     62.55,    39.22,    -,           -,           -           ])
    #tab52_06 = np.array([ 84.5,     60.5,     36.83,    -,           -,           -           ])
    #tab52_07 = np.array([ 114.6,    90.62,    65.85,    49.12 48.81, 37.84 37.56, 25.65 25.03 ])
    #tab52_08 = np.array([ -,        -,        -,        1.25 2.50,   1.0 1.5,     1.5 2.5     ])
    tab52_09  = np.array([ 0.261,    0.331,    0.444,    0.542,       0.632,       0.838       ])
    tab52_10  = np.array([ 0.000316, 0.000448, 0.000682, 0.0009015,   0.001035,    0.001093    ])
    #tab52_11 = np.array([ 0.000317, 0.000465, 0.000682, 0.000904,    -,           -           ])

    plt.plot(tab52_01, tab52_10, linestyle='', color='black', marker='+')

    # ------------------------------------------------------------------------

    plt.savefig(f'ref3937-Khyd-plot.pdf')

##############################################################################

def make_Hscc_plot():
    plt.figure()
    plt.xlabel('T')
    plt.ylabel('Hscc')

    # ------------------------------------------------------------------------

    # ref3942, tab on p. 208 and ref3925, Fig. 2:
    ref3942_temp = np.array([ 0, 15, 25, 39.1, 53.6, 67.6, 83.4 ])
    ref3942_Hscc = np.array([ 4.115,  2.065, 1.515, 1.003, 0.713, 0.552, 0.446 ])
    plt.plot(ref3942_temp, ref3942_Hscc, linestyle='', marker='o')

    # ------------------------------------------------------------------------

    # Tab. 53:
    # 1 строкв таблйцш 52 помещены temp опытовъ = row table 52 placed temperature experiments
    # 2 парциальное давление хлора = partial pressure of chlorine
    # 3 величины поглощения негидроливованнаго хлора = absorption values of non-hydrolyzed chlorine
    # 4 же величины, приведенныя къ нормальпому давленю = same values reduced to normal pressure
    # 5 величины теплоты = heat values (Delta H?)
    tab53_01 = np.array([ 15.,   25.,   39.1,  53.6,  67.6,  83.4  ])
    tab53_02 = np.array([ 736.4, 744.7, 711.1, 649.,  544.5, 348.1 ])
    tab53_03 = np.array([ 84.72, 60.6,  36.64, 22.65, 14.09, 6.979 ])
    tab53_04 = np.array([ 87.67, 61.81, 39.16, 26.52, 19.31, 15.20 ])
    tab53_05 = np.array([      6.6,   6.6,   6.0,   5.9,   4.4 ])

    print('check conversion to "reduced normal pressure", result should be 1:')
    print(760/tab53_02*tab53_03/tab53_04)

    # Tab. 53:
    # conversion from [mmol/(L*atm)] to dimless Hscc:
    conv = (tab53_01+CtoK) *Rgas / p0
    plt.plot(tab53_01, tab53_04*conv, linestyle='', marker='.')

    # ------------------------------------------------------------------------

    plt.savefig(f'ref3937-Hscc-plot.pdf')

##############################################################################

if __name__ == '__main__':

    make_Khyd_plot()
    make_Hscc_plot()
